--[[ Copyright (c) 2009 Peter "Corsix" Cawley

Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE. ]]

--[[ Miscellaneous utility functions ]]

local type, pairs, ipairs
    = type, pairs, ipairs

-- Reverse the order of an array
function table.reverse(t)
  local i, j = 1, #t
  while i < j do
    t[i], t[j] = t[j], t[i]
    i = i + 1
    j = j - 1
  end
  return t
end

-- Find an index of a given value within a given table
function table.indexof(t, v)
  for key, val in pairs(t) do
    if val == v then
      return key
    end
  end
end

-- Create an enumeration from a list
function enum(t)
  local e = {}
  local i = 0
  for _, entry in ipairs(t) do
    if type(entry) == "number" then
      i = entry
    else
      e[entry] = i
      e[i] = entry
      i = i + 1
    end
  end
  return e
end

-- Create a set from a list
function set(t)
  local s = {}
  for _, v in pairs(t) do
    s[v] = true
  end
  return s
end
